
# -------------------------------------------
# Set the CMAKE PARAMETERS
#  -------------------------------------------
MACRO(SET_CMAKE_PARAMETERS)
  SET(CMAKE_VERBOSE_MAKEFILE ON) # OFF
  SET(CMAKE_COLOR_MAKEFILE ON)   # to add Color on CMake Process 
ENDMACRO(SET_CMAKE_PARAMETERS)


#SET(DEBUG_MODE TRUE)

# --------------------------------------------------------------------------------
# General info first line
# --------------------------------------------------------------------------------
MACRO(MESSAGE_HEADER_LINE)
  MESSAGE("\n##--- POLYCHRONY INFO ---##--- POLYCHRONY INFO ---##--- POLYCHRONY INFO ---##---")
ENDMACRO(MESSAGE_HEADER_LINE)

# --------------------------------------------------------------------------------
# General info command line
# --------------------------------------------------------------------------------
MACRO(MESSAGE_PARAGRAPH_LINE CMD_STRING)
  MESSAGE("=> " ${CMD_STRING})
ENDMACRO(MESSAGE_PARAGRAPH_LINE)

# --------------------------------------------------------------------------------
# General info command line
# --------------------------------------------------------------------------------
MACRO(MESSAGE_SUB_LINE CMD_STRING)
  MESSAGE("   " ${CMD_STRING})
ENDMACRO(MESSAGE_SUB_LINE)

# --------------------------------------------------------------------------------
# General info last line
# --------------------------------------------------------------------------------
MACRO(GEN_MESSAGE_END)
  MESSAGE("# ------------------# ------------------# ------------------# ---------------- #")
ENDMACRO(GEN_MESSAGE_END)

# --------------------------------------------------------------------------------
# General info command line
# --------------------------------------------------------------------------------
MACRO(MESSAGE_COMMAND_LINE CMD_STRING)
  MESSAGE(${CMD_STRING})
ENDMACRO(MESSAGE_COMMAND_LINE)

# -------------------------------------------
# It prints arg in DEBUG MODE
# -------------------------------------------
MACRO(PRINT arg)
  IF(DEBUG_MODE)
    MESSAGE(${arg})
  ENDIF(DEBUG_MODE)
ENDMACRO(PRINT arg)

# -------------------------------------------
# It visits a directory (arg) if it exists.
# -------------------------------------------
MACRO(pK_ADD_SUBDIRECTORY arg)
  #  MESSAGE("PLG trace: entering pK_ADD_SUBDIRECTORY (${arg})")
  IF(EXISTS ${arg})
    SET(XX ${arg}/CMakeLists.txt)
    IF(EXISTS ${XX})
      #      MESSAGE("PLG trace: ${XX} found")
      #      GET_FILENAME_COMPONENT(ELEM ${arg} NAME)
      #      ADD_SUBDIRECTORY (${arg} ${arg})
      ADD_SUBDIRECTORY( ${arg} ) # "${CMAKE_CURRENT_BINARY_DIR}/${ELEM}")
    ENDIF(EXISTS ${XX})
  ELSE(EXISTS ${arg})
    PRINT("pK_ADD_SUBDIRECTORY --> directory not found ${arg}")
  ENDIF(EXISTS ${arg})
ENDMACRO(pK_ADD_SUBDIRECTORY arg)


# -------------------------------------------
# Managing tabulations
# The value of NB_TABUL= number of tab.
SET(NB_TABUL)

# -------------------------------------------
# Managing tabulations
# Increasing NB_TABUL
# -------------------------------------------
MACRO(INCR_TABUL)
  SET (NB_TABUL ${NB_TABUL} "-")
ENDMACRO(INCR_TABUL)

# -------------------------------------------
# Managing tabulations
# Decreasing NB_TABUL
# -------------------------------------------
MACRO(DECR_TABUL)
  LIST(LENGTH NB_TABUL N)
  PRINT("SIZE = ${N}")
  IF (N GREATER 0)
    LIST(REMOVE_AT NB_TABUL -1)
  ENDIF (N GREATER 0)
ENDMACRO(DECR_TABUL)

# -------------------------------------------
# Managing tabulations
# Getting in TAB th number of tabulations(TAB is a string of NB_TABUL tab)
# -------------------------------------------
MACRO(GET_STRTABULATIONS TAB)
  SET(TABULATION_STR)
  FOREACH(X ${NB_TABUL})
    SET(TABULATION_STR  "${TABULATION_STR} \t" )
  ENDFOREACH(X)
  SET(${TAB} ${TABULATION_STR})
ENDMACRO(GET_STRTABULATIONS TAB)

# ----------------------------------------------------------
# It generates in VFILE (a command file) the comment COM 
# ie: ${BEGIN_SHELL_COMMENT} ${COM} ${END_SHELL_COMMENT}
# ----------------------------------------------------------
MACRO(GEN_COMMENT VFILE COM)
  IF(PKCONFIG)
    FILE(APPEND ${VFILE} "${BEGIN_SHELL_COMMENT} ${COM} ${END_SHELL_COMMENT}\n")
  ENDIF(PKCONFIG)
ENDMACRO(GEN_COMMENT VFILE COM)

# ----------------------------------------------------------------------------------------
# It generates in VFILE the announce that the file VFILE is a generated file by CMAKE.
# It uses BC for the beginning of a comment and EC for the end of a comment.
# IF the global variable CURRENT_DEFS_CMAKE, that must contain the address of the source file 
# that produces the current command, is set, this adress is printed in the header.
# ----------------------------------------------------------------------------------------
MACRO(GEN_COMMENT_AUTOGEN1 VFILE BC EC)
  IF(PKCONFIG)
    FILE(APPEND ${VFILE} "${BC} ----------------------------------------------- ${EC}\n")
    FILE(APPEND ${VFILE} "${BC} This file has been generated by cmake. ${EC}\n")
    FILE(APPEND ${VFILE} "${BC} Manual editing not recommended. ${EC}\n")
    IF (DEFINED CURRENT_DEFS_CMAKE)
      FILE(APPEND ${VFILE} "${BC} Generated from ${CURRENT_DEFS_CMAKE} ${EC}\n")
    ENDIF (DEFINED CURRENT_DEFS_CMAKE)
    FILE(APPEND ${VFILE} "${BC} ----------------------------------------------- ${EC}\n")
    FILE(APPEND ${VFILE} "\n")
  ENDIF(PKCONFIG)
ENDMACRO( GEN_COMMENT_AUTOGEN1 VFILE  BC EM)

# --------------------------------------------------------------------------
# It calls the macro GEN_COMMENT_AUTOGEN1 for a command file VFILE
# --------------------------------------------------------------------------
MACRO(GEN_COMMENT_AUTOGEN VFILE)
  GEN_COMMENT_AUTOGEN1( ${VFILE} "${BEGIN_SHELL_COMMENT}" "${END_SHELL_COMMENT}")
ENDMACRO( GEN_COMMENT_AUTOGEN VFILE)

# -------------------------------------------------------------------------------------------------------
# It creates a shell variable ${a3}${a2} with value ${a1}/${a2} and 
# returns in RESULT the value TRUE if the operation success, FALSE otherwise.
#
# Example: with a1=Signal, a2=bin, a3=Signal_ the variable Signal_bin is created with Signal/bin as value.
# -------------------------------------------------------------------------------------------------------
MACRO(MKVARIABLE a1 a2 a3 RESULT)
  #PRINT("MKVARIABLE parameters: ${a1}; ${a2}; ${a3}")
  SET(${RESULT} FALSE)
  IF(NOT DEFINED ${a3}${a2})
    SET(XX ${a1}/${a2})
    IF(EXISTS ${XX})
      # For tcsh on MacOS/Solaris, the length of the shell variable name is limited.
      STRING(LENGTH ${a3}${a2} L)
      IF(L GREATER 30)
	MESSAGE(FATAL_ERROR "variable name too long:${a3}${a2} ... aborting")
      ENDIF(L GREATER 30)
      SET(${a3}${a2} ${XX})
      SET(${RESULT} TRUE)
      PRINT("Created Variable: ${a3}${a2}, value ${${a3}${a2}}")
    else(EXISTS ${XX})
      print("${a2} directory not found in ${a1}")
    ENDIF(EXISTS ${XX})
  ENDIF(NOT DEFINED ${a3}${a2})
ENDMACRO(MKVARIABLE)

# ------------------------------------------------------------------------------------------------------------------
# It defines and declares a set of shell variables.
# For a element foo of DIRS (a list of directories), a shell variable PREFIXfoo is created with DIRFROM/foo as value.
# And this variable is declared in the CONFIGURATION file of the a directory WHERETODECLARE
# ------------------------------------------------------------------------------------------------------------------
MACRO(SETVARIABLES DIRS DIRFROM PREFIX WHERETODECLARE)
  SET(B FALSE)
  FOREACH(aDir ${DIRS})
    FILE(MAKE_DIRECTORY "${DIRFROM}/${aDir}")
    MKVARIABLE(${DIRFROM} ${aDir} ${PREFIX} B)
  ENDFOREACH(aDir)
  MKCONFIG("${DIRS}" ${DIRFROM} ${PREFIX} ${WHERETODECLARE})
ENDMACRO(SETVARIABLES)

#-------------------------------------------------------------------------
# It creates the directory DIRFROM/cmake, if it does not exist in DIRFROM.
# ------------------------------------------------------------------------
MACRO(pK_MK_CMAKE_DIR DIRFROM)
  SET(YY ${DIRFROM}/CMakeLists.txt)
  IF(EXISTS ${YY})  
    SET(YY ${DIRFROM}/cmake)
    MK_DIR(${YY})
    #     IF(NOT EXISTS ${YY})
    #       FILE(MAKE_DIRECTORY ${YY})
    #     ELSE(NOT EXISTS ${YY})
    #       IF(NOT IS_DIRECTORY ${YY})
    # 	MESSAGE(FATAL_ERROR "${YY} is not a directory!!!")
    #       ENDIF(NOT IS_DIRECTORY ${YY})
    #     ENDIF(NOT EXISTS ${YY})

  ENDIF(EXISTS ${YY})  
ENDMACRO(pK_MK_CMAKE_DIR DIRFROM)

# ---------------------------------------------------------
# If the directory DIRNAME doest not exist, it is created. 
# Otherwise, it verifies that DIRNAME is a directory name.
# ---------------------------------------------------------
MACRO(MK_DIR DIRNAME)
  IF(NOT EXISTS ${DIRNAME})
    #    MESSAGE("MAKE_DIRECTORY =${DIRNAME}")
    FILE(MAKE_DIRECTORY ${DIRNAME})
  ELSE(NOT EXISTS ${DIRNAME})
    IF(NOT IS_DIRECTORY ${DIRNAME})
      MESSAGE(FATAL_ERROR "${DIRNAME} is not a directory!!!")
    ENDIF(NOT IS_DIRECTORY ${DIRNAME})
  ENDIF(NOT EXISTS ${DIRNAME})
ENDMACRO(MK_DIR DIRNAME)

MACRO(LOAD_USER_CMAKE_DEFS DIRFROM)
  SET(XX ${DIRFROM}/${PREDEF_DEFS_FILENAME_CMAKE})
  PRINT(">>>>>>>>>>>>>>> DECLARE_COMPONENT XX : ${XX}")
  IF(EXISTS ${XX})
    IF(PKCONFIG)
      GET_STD_CONFIG_NAME(${DIRFROM} CONFIG_FILE)
      FILE(REMOVE ${CONFIG_FILE})
      GET_CONFIGREC_NAME(${DIRFROM} CONFIGREC)
      FILE(REMOVE ${CONFIGREC})
    ENDIF(PKCONFIG)
    INCLUDE(${XX})
    IF(PKCONFIG)
      GET_STD_CONFIG_NAME(${DIRFROM} CONFIG_FILE)
      GET_CONFIGREC_NAME(${DIRFROM} CONFIGREC)
      MERGE_CONFIG_FILES(${CONFIG_FILE} ${CONFIGREC})
    ENDIF(PKCONFIG)
  ENDIF(EXISTS ${XX})
ENDMACRO(LOAD_USER_CMAKE_DEFS DIRFROM)

#----------------------------------------------------------------------
# It loads/includes the CMAKE file defs.cmake of the directory DIRFROM.
# ---------------------------------------------------------------------
MACRO(DECLARE_COMPONENT DIRFROM)
  IF(NOT DEFINED ${DIRFROM})
    LOAD_USER_CMAKE_DEFS(${DIRFROM})
    pK_MK_CMAKE_DIR(${DIRFROM})
    SET(${DIRFROM} DEFINED)
  ENDIF(NOT DEFINED ${DIRFROM}) 
ENDMACRO(DECLARE_COMPONENT)

MACRO(DECLARECOMPONENT DIRFROM)
  DECLARE_COMPONENT(${DIRFROM})
ENDMACRO(DECLARECOMPONENT DIRFROM)

MACRO(MERGE_CONFIG_FILES CONFIG1 CONFIG2)
  IF(EXISTS ${CONFIG2})
    FILE(READ ${CONFIG2} vfile)
    FILE(APPEND ${CONFIG1} ${vfile})
    FILE(REMOVE ${CONFIG2})
  ENDIF(EXISTS ${CONFIG2})
ENDMACRO(MERGE_CONFIG_FILES CONFIG1 CONFIG2)

#---------------------------------------------------------------------------------------
# For each element FOO of a list (DIRS), it calls DECLARE_COMPONENT DIRFROM(DIRFROM/FOO)
# --------------------------------------------------------------------------------------
MACRO(DECLARECOMPONENTS DIRS DIRFROM)
  IF(NOT DEFINED ${DIRFROM})
    INCR_TABUL()
    FOREACH(aDir ${DIRS})
      DECLARE_COMPONENT("${${DIRFROM}${aDir}}")
    ENDFOREACH(aDir)
    SET(${DIRFROM} DEFINED)
    DECR_TABUL()
  ENDIF(NOT DEFINED ${DIRFROM})
ENDMACRO(DECLARECOMPONENTS)

# ---------------------------------------------------------------------------------------
# It adds the suffix of a command file to NAME and returns the value in CONFIG_FILE.
# example: on Windows, MK_CONFIG_NAME FOO CONFIG_FILE) will return FOO.bat in CONFIG_FILE.
# ---------------------------------------------------------------------------------------
MACRO (MK_CONFIG_NAME NAME CONFIG_FILE)
  IF(UNIX)
    SET(${CONFIG_FILE} ${NAME})
  ELSE(UNIX) # windows
    SET(${CONFIG_FILE} ${NAME}.bat)
  ENDIF(UNIX)
ENDMACRO (MK_CONFIG_NAME NAME CONFIG_FILE)

MACRO(PK_FIND_COMMAND_UNIX NAMEVAR CMD)
  IF(UNIX)
    FIND_PROGRAM(${NAMEVAR} ${CMD})
    IF(${NAMEVAR})
    ELSE(${NAMEVAR})
      MESSAGE(FATAL_ERROR "${CMD} not found")
    ENDIF(${NAMEVAR})
  ENDIF(UNIX)
ENDMACRO(PK_FIND_COMMAND_UNIX NAMEVAR CMD)

# --------------------------------------------------------------------------------------------------
# It defines the symbols (BEGIN_SHELL_COMMENT, END_SHELL_COMMENT) used by the shell for the comments
# --------------------------------------------------------------------------------------------------
MACRO(SET_COMMENTS_SHELL)
  SET(END_SHELL_COMMENT)
  IF(UNIX)
    SET (BEGIN_SHELL_COMMENT "#")
  ELSE(UNIX) # windows
    SET (BEGIN_SHELL_COMMENT "rem")
  ENDIF(UNIX)
ENDMACRO(SET_COMMENTS_SHELL)

# --------------------------------------------------------------------------------------------------
# Initializing : defining the name of the config files.
# --------------------------------------------------------------------------------------------------
MACRO(INIT_CONFIG)
  MK_CONFIG_NAME(${PREDEF_PKCONFIG_NAME} PK_CONFIG_FILE)
  MK_CONFIG_NAME(${PREDEF_PKCONFIG_REC_NAME} PK_CONFIG_FILE_REC)
ENDMACRO(INIT_CONFIG)

# --------------------------------------------------------------------------------------------------
# It defines the symbols (BEGIN_SHELL_COMMENT, END_SHELL_COMMENT) used by the shell for the comments
# It verifies that tcsh, sh are available (Unix): the cmake variables are resp. TCSH, THESH.
# --------------------------------------------------------------------------------------------------
MACRO(SET_SHELL_DEFS)
  SET_COMMENTS_SHELL()
  PK_FIND_COMMAND_UNIX(TCSH tcsh)
  PK_FIND_COMMAND_UNIX(THESH sh)
ENDMACRO(SET_SHELL_DEFS)


# ---------------------------------------------------------------------------
# It returns in  REF_SHVAR the reference to a shell variable SHVAR.
# $SHVAR pour tcsh, %SHVAR% pour script windows.
# ---------------------------------------------------------------------------
MACRO(MAKE_REF_SHELLVAR SHVAR REF_SHVAR)
  IF (UNIX)
    SET(${REF_SHVAR} "\$${SHVAR}")
  else (UNIX)
    SET(${REF_SHVAR} "%${SHVAR}%")
  endif (UNIX)
ENDMACRO(MAKE_REF_SHELLVAR SHVAR REF_SHVAR)

# ---------------------------------------------------------------------------
#  It returns in  REF_SHVAR the value of SHVAR followed by the path separator.
# ---------------------------------------------------------------------------
MACRO(ADD_PATH_SEPARATOR SHVAR REF_SHVAR)
  IF (UNIX)
    SET(${REF_SHVAR} "${SHVAR}/")
  ELSE (UNIX)
    SET(${REF_SHVAR} "${SHVAR}\\\\")
  ENDIF (UNIX)
ENDMACRO(ADD_PATH_SEPARATOR SHVAR REF_SHVAR)

# --------------------------------------------------------------------------
# Defining the variable RES by STR1/STR2 (Linux) STR1\STR2 (Windows)
# --------------------------------------------------------------------------
MACRO(CONC_STR_PATH STR1 STR2 RES)
  #ADD_PATH_SEPARATOR(${STR1} ${RES})
  SET(${RES} "${STR1}/${STR2}")
ENDMACRO(CONC_STR_PATH STR1 STR2 RES)

# ---------------------------------------------------------------------------
# It adds to the file FILENAME the definition DEF of a shell variable SHVAR.
# The definition is prefixed by the string PREFIX.
# ---------------------------------------------------------------------------
MACRO(ADD_SHELL_VAR FILENAME SHVAR DEF PREFIX)
  IF(PKCONFIG)
    IF(UNIX)
      SET(VSET "setenv ${SHVAR} ${DEF}")
    ELSEIF(WIN32)
      SET(VSET "set ${SHVAR}=${DEF}")
    ENDIF(UNIX)
    FILE(APPEND ${FILENAME} "${PREFIX} ${VSET}\n")
  ENDIF(PKCONFIG)
ENDMACRO(ADD_SHELL_VAR FILENAME SHVAR DEF PREFIX)



#--------------------------------------------------------------------------------------------------------
# It generates in a file NAME_FILE_CONFIG the loading of a script file SH_VAR/PK_CONFIG_FILE.
# 
# Example: (linux) ADD_LOAD_SETENV( F pK_com) will generate in F the following lines 
#           where PK_CONFIG_FILE is defined by _config_:
#                        if ( -ef "${pK_com}/_config_") then
#                           source ${pK_com}/_config_
#                        endif
#--------------------------------------------------------------------------------------------------------
MACRO(ADD_LOAD_SETENV NAME_FILE_CONFIG SH_VAR )
  IF(PKCONFIG)
    #    MESSAGE (" NAME_FILE_CONFIG = ${NAME_FILE_CONFIG}, SH_VAR = ${SH_VAR} ")
    IF (UNIX)
      FILE(APPEND ${NAME_FILE_CONFIG} " if ( -ef \"\${${SH_VAR}}/${PK_CONFIG_FILE}\") then\n")
      FILE(APPEND ${NAME_FILE_CONFIG} "    source \${${SH_VAR}}/${PK_CONFIG_FILE}\n")
      FILE(APPEND ${NAME_FILE_CONFIG} " endif\n")
    ELSE (UNIX)
      FILE(APPEND ${NAME_FILE_CONFIG} " if exist \"%${SH_VAR}%\\${PK_CONFIG_FILE}\"  call \"%${SH_VAR}%\\${PK_CONFIG_FILE}\"\n")
    ENDIF (UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(ADD_LOAD_SETENV NAME_FILE_CONFIG SH_VAR)

# -----------------------------------------------------------------------------------------
# It adds to the file CONFIG_FILE the definition of a shell variable named ${PREFIX_NAME}${SUBDIR} 
# by the value ${DIR}/${SUBDIR}.
#   DIR: directory path,
#   SUBDIR: sub-directory of a1,
#   PREFIX_NAME: prefix of the name of the variable defining a1,
#   CONFIG_FILE: filename config.
#   FATHERNAMEDIR: the directory where to declare this variable.
#
# example: MK_CONFIG_VARIABLE(Signal ${pK_ROOT} pK_  _config_ pK_ROOT ) 
#    will generate the following definition 
#       - for Linux,   setenv pK_Signal $pK_ROOT/Signal in the file ${pK_ROOT}/_config_
#       - for Windows, set pK_Signal=%pK_ROOT%\Signal in the file ${pK_ROOT}\_config_.bat
# ----------------------------------------------------------------------------------------
MACRO(MK_CONFIG_VARIABLE DIR SUBDIR PREFIX_NAME CONFIG_FILE FATHERNAMEDIR FOO)
  SET(XX ${DIR}/${SUBDIR})
  IF(EXISTS ${XX})
    SET(${PREFIX_NAME}${SUBDIR} ${XX})
    MAKE_REF_SHELLVAR(${FATHERNAMEDIR} NAME)
    ADD_PATH_SEPARATOR(${NAME} NAME)
    ADD_SHELL_VAR(${CONFIG_FILE} ${PREFIX_NAME}${SUBDIR} "${NAME}${SUBDIR}" "")
    #    ADD_LOAD_SETENV(${CONFIG_FILE} ${PREFIX_NAME}${SUBDIR})
    ADD_LOAD_SETENV(${FOO} ${PREFIX_NAME}${SUBDIR})
  ELSE(EXISTS ${XX})
    PRINT("${SUBDIR} directory not found in ${DIR}")
  ENDIF(EXISTS ${XX})
ENDMACRO(MK_CONFIG_VARIABLE)

#---------------------------------------------------------------------------
# It generates the header of a script file FILE, in silent mode (no message).
# The GEN_COMMENT_AUTOGEN() command is called.
#---------------------------------------------------------------------------
MACRO(GEN_HEADER_SCRIPT_SILENT  FILE )
  IF (UNIX)
    FILE(WRITE ${FILE} "${BEGIN_SHELL_COMMENT}!${TCSH}\n")
  ELSE(UNIX)
    FILE(WRITE ${FILE} "echo off \n")
  ENDIF(UNIX)
  GEN_COMMENT_AUTOGEN(${FILE}) # this file is auto generated....
  ADD_EXECUTE_RIGHT(${FILE})
ENDMACRO(GEN_HEADER_SCRIPT_SILENT  FILE)

#----------------------------------------------------------------
# Similar to GEN_HEADER_SCRIPT_SILENT, in verbose mode: the 
# message "Generating file: ${FILE}" is printed.
#----------------------------------------------------------------
MACRO(GEN_HEADER_SCRIPT  FILE)
  FILE(TO_NATIVE_PATH ${FILE} XOS)
  MESSAGE(" ${XOS} updated")
  GEN_HEADER_SCRIPT_SILENT(${FILE})
ENDMACRO(GEN_HEADER_SCRIPT  FILE)

#------------------------------------------------------------------------------------------------------
# It generates in the file VFILE the requirement of a shell variable SHVAR.
#
# Example: GEN_REQUIRE_SHELL_VAR(F utl_Host) will generate in the file F the following code (for Linux)
# if ( ! $?utl_Host ) then 
#    echo "##  ERROR  ## utl_Host not defined"
#    echo "  run <source PolychronyToolset_setup> in PolychronyToolset root directory"
#    exit
#  endif
#------------------------------------------------------------------------------------------------------
MACRO(GEN_REQUIRE_SHELL_VAR VFILE SHVAR)
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${VFILE} " if ( ! \$?${SHVAR} ) then \n")
      FILE(APPEND ${VFILE} "   echo \"##  ERROR  ## ${SHVAR} not defined\"\n")
      FILE(APPEND ${VFILE} "   echo \"  run <source ${pkEnvFile}> in ${SOFTNAME} root directory\"\n")
      FILE(APPEND ${VFILE} "   exit -1\n")
      FILE(APPEND ${VFILE} " endif\n")
      FILE(APPEND ${VFILE} "\n")
    ELSE(UNIX) #WIN32
      FILE(APPEND ${VFILE} " if not defined ${SHVAR} (\n")
      FILE(APPEND ${VFILE} "   echo \"##  ERROR  ## ${SHVAR} not defined\"\n")
      FILE(APPEND ${VFILE} "   echo \"  run ${pkEnvFile}.bat in ${SOFTNAME} root directory\"\n")
      FILE(APPEND ${VFILE} "   goto :eof\n")
      FILE(APPEND ${VFILE} " )\n")
      FILE(APPEND ${VFILE} "\n")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_REQUIRE_SHELL_VAR VFILE SHVAR)

MACRO(GEN_ECHO VFILE MESS)
  FILE(APPEND ${VFILE} "echo \"${MESS}\"\n")
  FILE(APPEND ${VFILE} "\n")
ENDMACRO(GEN_ECHO VFILE MESS)

#------------------------------------------------------------------------------------------------------
# It generates in the file VFILE the loading of the ${pkEnvFile} file if the shell variable SHVAR is not known.
#------------------------------------------------------------------------------------------------------
MACRO(GEN_REQUIRE_SHELL_VAR_LOADENV VFILE SHVAR)
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${VFILE} " if ( ! \$?${SHVAR} ) then \n")
      FILE(APPEND ${VFILE} "   source ${pK_ROOT}/${pkEnvFile}\n")
      FILE(APPEND ${VFILE} " endif\n")
      FILE(APPEND ${VFILE} "\n")
    ELSE(UNIX) #WIN32
      FILE(APPEND ${VFILE} " if not defined ${SHVAR} (\n")
      FILE(APPEND ${VFILE} "   call ${pK_ROOT}\\${pkEnvFile}\n")
      FILE(APPEND ${VFILE} " )\n")
      FILE(APPEND ${VFILE} "\n")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_REQUIRE_SHELL_VAR_LOADENV VFILE SHVAR)

#------------------------------------------------------------------------------------------------------
# It generates in the file VFILE the definition of a shell variable SHVAR, if this variable is not 
# already defined. This variable is defined by he value VALUE. A comment prefixes this definition.
#
# Example: 
#  GEN_SHELL_VAR_IF_NOTDEFINED(F T_MEM_ARB) will generate in the file F the following code (for Linux)
#   # Tree memory parameter (optional)
#   if ( ! $?T_MEM_ARB ) then 
#        setenv T_MEM_ARB 60000000 
#   endif
#------------------------------------------------------------------------------------------------------
MACRO(GEN_SHELL_VAR_IF_NOTDEFINED VFILE SHVAR VALUE COM)
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${VFILE} "# ${COM}")
      FILE(APPEND ${VFILE} " if ( ! \$?${SHVAR} ) then \n")
      ADD_SHELL_VAR(${VFILE} ${SHVAR} ${VALUE} "     ")
      FILE(APPEND ${VFILE} " endif\n")
    ELSE(UNIX)
      FILE(APPEND ${VFILE} "rem ${COM}")
      ADD_SHELL_VAR(${VFILE} ${SHVAR} ${VALUE} "     ")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_SHELL_VAR_IF_NOTDEFINED VFILE SHVAR VALUE COM)

#--------------------------------------------------------------------------------
# It generates in the file VFILE, the test of the existence of the file DIR/NFILE
#--------------------------------------------------------------------------------
MACRO(GEN_REQUIRE_FILE VFILE DIR NFILE )
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${VFILE} " if (! -efr \"\${${DIR}}/${NFILE}\") then \n")
      FILE(APPEND ${VFILE} "    echo \" ERROR : cannot read file ${NFILE} in \${${DIR}}\"\n")
      FILE(APPEND ${VFILE} "    exit -1\n")
      FILE(APPEND ${VFILE} " endif\n")
    ELSE(UNIX)
      FILE(APPEND ${VFILE} " if not exist \"%${DIR}%\\${NFILE}\" ( \n")
      FILE(APPEND ${VFILE} "    @echo \" ERROR : cannot read file ${NFILE} in %${DIR}%\"\n")
      FILE(APPEND ${VFILE} "    goto :eof\n")
      FILE(APPEND ${VFILE} " )\n")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_REQUIRE_FILE VFILE DIR NFILE )


#--------------------------------------------------------------------------------
# It generates in the file VFILE, the test of the existence of the directory DIR
#--------------------------------------------------------------------------------
MACRO(GEN_REQUIRE_DIR VFILE DIR )
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${VFILE} " if (! -edr \"\${${DIR}}\") then \n")
      FILE(APPEND ${VFILE} "    echo \" ERROR : the directory \${${DIR}} does not exist\"\n")
      FILE(APPEND ${VFILE} "    exit -1\n")
      FILE(APPEND ${VFILE} " endif\n")
    ELSE(UNIX)
      FILE(APPEND ${VFILE} " if not exist \"%${DIR}%\" ( \n")
      FILE(APPEND ${VFILE} "    @echo \" ERROR : the directory %${DIR}% does not exist\"\n")
      FILE(APPEND ${VFILE} "    goto :eof\n")
      FILE(APPEND ${VFILE} " )\n")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_REQUIRE_DIR VFILE DIR )

# -----------------------------------------------------------------
# It generates in the file FILE the execution mode (verbose or not)
# Verbose mode is set when DEBUG_MODE.
# -----------------------------------------------------------------
MACRO(GEN_ECHO_MODE FILE)
  IF(PKCONFIG)
    IF(UNIX)
      
    ELSE(UNIX) #WIN32
      IF(DEBUG_MODE)
	FILE(APPEND ${FILE} "@echo ON\n")
      ELSE(DEBUG_MODE)
	FILE(APPEND ${FILE} "@echo OFF\n")
      ENDIF(DEBUG_MODE)
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_ECHO_MODE FILE)

# -------------------------------------------------------------------------
# It generates in the file FILE the command that prints the message MESSAGE
# -------------------------------------------------------------------------
MACRO(GEN_ECHO FILE MESSAGE)
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${FILE} "echo ")
    ELSE(UNIX) #WIN32
      FILE(APPEND ${FILE} "@echo ")
    ENDIF(UNIX)
    FILE(APPEND ${FILE} "${MESSAGE}\n")
  ENDIF(PKCONFIG)
ENDMACRO(GEN_ECHO FILE MESSAGE)


# ------------------------------------------------------------------------------------
# It generates in the file FILE the message MESS in verbose mode (set when DEBUG_MODE)
# ------------------------------------------------------------------------------------
MACRO(GEN_MESSAGE_DBG_MODE FILE MESS)
  IF(PKCONFIG)
    IF(DEBUG_MODE)
      GEN_ECHO(${FILE} ${MESS})
    ENDIF(DEBUG_MODE)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_MESSAGE_DBG_MODE FILE MESS)

#-----------------------------------------------------------------------------
# It generates the header of a script file FILE:
#  - it calls GEN_HEADER_SCRIPT()
#  - it sets the mode for the echo command (useful for debugging).
#-----------------------------------------------------------------------------
MACRO(BEGIN_CONFIG_FILE NAME_FILE_CONFIG)
  GEN_HEADER_SCRIPT(${NAME_FILE_CONFIG})
  GEN_ECHO_MODE(${NAME_FILE_CONFIG})
  GEN_MESSAGE_DBG_MODE(${NAME_FILE_CONFIG} " Loading: ${NAME_FILE_CONFIG}")
ENDMACRO(BEGIN_CONFIG_FILE NAME_FILE_CONFIG)

#-----------------------------------------------------------------------------
# It finalizes the generation of a script file FILE.
#-----------------------------------------------------------------------------
MACRO(END_CONFIG_FILE NAME_FILE_CONFIG)

ENDMACRO(END_CONFIG_FILE NAME_FILE_CONFIG)


MACRO(GET_CONFIG_FILE DIRFROM NAME_FILE_CONFIG)
  IF(DEFINED GLOBAL_CONFIG_FILE)
    SET(NAME_FILE_CONFIG ${GLOBAL_CONFIG_FILE})
  ELSE(DEFINED  GLOBAL_CONFIG_FILE)
    SET(NAME_FILE_CONFIG ${DIRFROM}/${PK_CONFIG_FILE})
  ENDIF(DEFINED GLOBAL_CONFIG_FILE)
ENDMACRO(GET_CONFIG_FILE DIRFROM NAME_FILE_CONFIG)

MACRO(GET_CONFIG_FILE_REC DIRFROM NAME_FILE_CONFIG_REC)
  SET(NAME_FILE_CONFIG_REC ${DIRFROM}/${PK_CONFIG_FILE_REC})
ENDMACRO(GET_CONFIG_FILE_REC DIRFROM NAME_FILE_CONFIG_REC)
#-----------------------------------------------------------------------------
# It returns in $NAME_FILE_CONFIG the name of the current config file. 
# If the symbol GLOBAL_CONFIG_FILE is defined it takes its value, 
# otherwise if the given by ${DIRFROM}/${PK_CONFIG_FILE}
#-----------------------------------------------------------------------------
MACRO(GET_CONFIG_FILES DIRFROM NAME_FILE_CONFIG NAME_FILE_CONFIG_REC)
  GET_CONFIG_FILE(${DIRFROM} ${NAME_FILE_CONFIG})
  GET_CONFIG_FILE_REC(${DIRFROM} ${NAME_FILE_CONFIG_REC})
ENDMACRO(GET_CONFIG_FILES DIRFROM NAME_FILE_CONFIG B_FILE_CONFIG)


MACRO(DEFINE_ROOT_SYMBOL NAME_FILE_CONFIG PREFIX NAME)
  IF(NOT DEFINED ${PREFIX}ROOT)
    ADD_SHELL_VAR(${NAME_FILE_CONFIG} ${PREFIX}ROOT ${NAME} "")
    SET(${PREFIX}ROOT ${NAME})
  ENDIF(NOT DEFINED ${PREFIX}ROOT)
ENDMACRO(DEFINE_ROOT_SYMBOL NAME_FILE_CONFIG PREFIX NAME)

#--------------------------------------------------------------------------------------------------------
# It generates a configuration file DIRFROM/PK_CONFIG_FILE, where PK_CONFIG_FILE is a global name.
# when this file doest not exist or when FORCED.
# In this file, it generates for each element FOO of DIRS the declaration of a shell variable,
# using the command MK_CONFIG_VARIABLE()
#--------------------------------------------------------------------------------------------------------
MACRO(MKCONFIG DIRS DIRFROM PREFIX FATHERNAMEDIR)
  GET_CONFIG_FILES(${DIRFROM} NAME_FILE_CONFIG NAME_FILE_CONFIG_REC) 
  MAKE_REF_SHELLVAR(${FATHERNAMEDIR} NAME)
  GEN_COMMENT(${NAME_FILE_CONFIG}  ${PREFIX})

  DEFINE_ROOT_SYMBOL(${NAME_FILE_CONFIG} ${PREFIX} ${NAME})

  FOREACH(aDir ${DIRS})
    MK_CONFIG_VARIABLE(${DIRFROM} ${aDir} ${PREFIX} ${NAME_FILE_CONFIG} ${FATHERNAMEDIR} ${NAME_FILE_CONFIG_REC})
  ENDFOREACH(aDir)
  #  ADD_UNDEFINED_STD_VAR()
  END_CONFIG_FILE(${NAME_FILE_CONFIG})
  
ENDMACRO(MKCONFIG DIRS DIRFROM PREFIX FATHERNAMEDIR)

MACRO(MAKE_WINDOWS_PATH pathname)
  # An extra \\ escape is necessary to get a \ through CMake's processing.
  STRING(REPLACE "/" "\\" ${pathname} "${${pathname}}")
  # Enclose with UNESCAPED quotes.  This means we need to escape our
  # quotes once here, i.e. with \"
  SET(${pathname} \"${${pathname}}\")
ENDMACRO(MAKE_WINDOWS_PATH)




# -----------------------------------------------------------------------------
# It returns in OSTR the path ISTH for the current OS.
# CMAKE works with the Linux syntax. So, for Windows it is useful to modify 
# it for the configuration file.
# -----------------------------------------------------------------------------
MACRO(MK_SYSTEM_PATH ISTR OSTR)
  IF(UNIX)
    SET(${OSTR} ${ISTR})
  ELSE(UNIX)
    STRING(REPLACE / \\\\  ${OSTR} "${ISTR}")
  ENDIF(UNIX)
ENDMACRO(MK_SYSTEM_PATH ISTR OSTR)

# -----------------------------------------------------------------------------
# It adds the "execute" right to a file FILE
# -----------------------------------------------------------------------------
MACRO(ADD_EXECUTE_RIGHT FILE)
  IF(UNIX)
    IF(EXISTS ${FILE})
      EXECUTE_PROCESS(COMMAND chmod uo+rx ${FILE})
    ENDIF(EXISTS ${FILE})
  ENDIF(UNIX)
ENDMACRO(ADD_EXECUTE_RIGHT FILE})

# -----------------------------------------------------------------------------
# It returns the representation of a CMAKE list (L) as a CMAKE string (STRL).
# -----------------------------------------------------------------------------
MACRO (LISTOPTIONS2STRING L STRL)
  SET(OPTIONS_STRING)
  FOREACH(ELEM ${L})
    SET(OPTIONS_STRING "${OPTIONS_STRING} ${ELEM}")
  ENDFOREACH(ELEM)
  #  STRING(REPLACE " " "|" YYY ${OPTIONS_STRING})
  SET(${STRL} ${OPTIONS_STRING})
ENDMACRO(LISTOPTIONS2STRING L STRL)

# -------------------------------------------------------------------------------
# It generates in the file CONFIG_FILE the definition of a command CMD 
# parameterized by the options OPTIONS and its default option is DEFAULT_OPTIONS.
# The CONFIG_FILE is generated in the directory WHERE.
# -------------------------------------------------------------------------------
MACRO(GEN_CMAKE_CMD CONFIG_FILE OPTIONS DEFAULT_OPTIONS CMD WHERE TYPE)
  IF(PKCONFIG)
    LISTOPTIONS2STRING("${OPTIONS}" STROPTIONS)
    STRING(COMPARE EQUAL  ${TYPE} "STD" IS_STD)
    STRING(COMPARE EQUAL  ${TYPE} "EXAMPLE" IS_EXAMPLE)
    LIST(LENGTH OPTIONS NB)
    IF(UNIX)
      FILE(APPEND ${CONFIG_FILE} " if ($# > 1) then\n")
      FILE(APPEND ${CONFIG_FILE} "    echo \"Wrong number of parameter, usage: ${CMD} [${STROPTIONS}]\"\n")
      FILE(APPEND ${CONFIG_FILE} "    exit -1\n")
      FILE(APPEND ${CONFIG_FILE} " endif\n")
      FILE(APPEND ${CONFIG_FILE} " if ($# == 0) then\n")
      FILE(APPEND ${CONFIG_FILE} "    set arguments=\"${DEFAULT_OPTIONS}\"\n")
      FILE(APPEND ${CONFIG_FILE} " else\n")
      
      IF(NB GREATER 0)
	FILE(APPEND ${CONFIG_FILE} " if ")
	IF(NB GREATER 1)
	  FILE(APPEND ${CONFIG_FILE} "( ")
	ENDIF(NB GREATER 1)
	FOREACH(I RANGE 1 ${NB})
	  MATH(EXPR CUR "${I}-1")
	  LIST(GET OPTIONS ${CUR} OPT)
	  FILE(APPEND ${CONFIG_FILE} "(\"\$1\" != \"${OPT}\")")
	  IF(${I} LESS NB)
	    FILE(APPEND ${CONFIG_FILE} " && ")
	  ENDIF(${I} LESS NB)
	ENDFOREACH(I RANGE 1 N)
	IF(NB GREATER 1)
	  FILE(APPEND ${CONFIG_FILE} " )")
	ENDIF(NB GREATER 1)
	FILE(APPEND ${CONFIG_FILE} "    then\n")
	FILE(APPEND ${CONFIG_FILE} "       echo \"improper parameter: $1, usage: ${CMD} [${STROPTIONS}]\"\n")
	FILE(APPEND ${CONFIG_FILE} "       exit -1\n")
	FILE(APPEND ${CONFIG_FILE} "    else \n")
	FILE(APPEND ${CONFIG_FILE} "      set arguments=\$1\n")
	FILE(APPEND ${CONFIG_FILE} "    endif\n")
	FILE(APPEND ${CONFIG_FILE} " endif\n")
	FILE(APPEND ${CONFIG_FILE} "\n")
      ENDIF(NB GREATER 0)
      
      FILE(APPEND ${CONFIG_FILE} " set xx=\"\$PWD\"\n")
      FILE(APPEND ${CONFIG_FILE} " cd \${${WHERE}}/cmake \n")
      FILE(APPEND ${CONFIG_FILE} " if (! -efr \"CMakeCache.txt\") then \n")
      FILE(APPEND ${CONFIG_FILE} "    cmake .. -D${CMD}=1\n")
      FILE(APPEND ${CONFIG_FILE} " endif\n")
      
      FILE(APPEND ${CONFIG_FILE} " if ( ! \$?MAKE_OPTIONS ) then \n")
      FILE(APPEND ${CONFIG_FILE} "  setenv MAKE_OPTIONS \"-s -i\" \n")
      FILE(APPEND ${CONFIG_FILE} " endif\n")
      FILE(APPEND ${CONFIG_FILE} "\n")
      IF(IS_STD)
	FILE(APPEND ${CONFIG_FILE} " if (\"\${arguments}\" == \"force\" ) then \n")
	FILE(APPEND ${CONFIG_FILE} "  make src_clean \${MAKE_OPTIONS}\n")
	FILE(APPEND ${CONFIG_FILE} "  make clean \${MAKE_OPTIONS}\n")
	FILE(APPEND ${CONFIG_FILE} "  make source \${MAKE_OPTIONS}\n")
	FILE(APPEND ${CONFIG_FILE} "  make \${MAKE_OPTIONS}\n")
	FILE(APPEND ${CONFIG_FILE} " else\n")
	FILE(APPEND ${CONFIG_FILE} "  make \${arguments} \${MAKE_OPTIONS}\n") 
	FILE(APPEND ${CONFIG_FILE} " endif\n")
      ELSEIF(IS_EXAMPLE) 
        FILE(APPEND ${CONFIG_FILE} "  make \${arguments} \${MAKE_OPTIONS}\n") 
      ENDIF(IS_STD) 
      FILE(APPEND ${CONFIG_FILE} " cd \"\$xx\" \n")
      FILE(APPEND ${CONFIG_FILE} " unset xx \n")
      
    ELSE(UNIX)
      # WINDOWS
      FILE(APPEND ${CONFIG_FILE} " if not \"%2%\" == \"\" (\n")
      FILE(APPEND ${CONFIG_FILE} "    @echo \"Wrong number of parameter, usage: ${CMD} [${STROPTIONS}]\"\n")
      FILE(APPEND ${CONFIG_FILE} "    goto :eof\n")
      FILE(APPEND ${CONFIG_FILE} " )\n")
      FILE(APPEND ${CONFIG_FILE} " if \"%1%\" == \"\" (\n")
      FILE(APPEND ${CONFIG_FILE} "    set arguments=\"${DEFAULT_OPTIONS}\"\n")
      FILE(APPEND ${CONFIG_FILE} "    goto fixedarg\n")
      FILE(APPEND ${CONFIG_FILE} " ) else (\n")
      IF(NB GREATER 0)
	
	FOREACH(I RANGE 1 ${NB})
	  MATH(EXPR CUR "${I}-1")
	  LIST(GET OPTIONS ${CUR} OPT)
	  FILE(APPEND ${CONFIG_FILE} " if \"%1%\" == \"${OPT}\" goto okarg\n")
	ENDFOREACH(I RANGE 1 N)
	IF(NB GREATER 1)
	  FILE(APPEND ${CONFIG_FILE} " )\n")
	ENDIF(NB GREATER 1)	
	FILE(APPEND ${CONFIG_FILE} " @echo \"improper parameter: %1%, usage: ${CMD} [${STROPTIONS}]\"\n")
	FILE(APPEND ${CONFIG_FILE} " goto :eof\n")
	FILE(APPEND ${CONFIG_FILE} " ")
	FILE(APPEND ${CONFIG_FILE} " :okarg\n")
	FILE(APPEND ${CONFIG_FILE} " set arguments=%1%\n")
	FILE(APPEND ${CONFIG_FILE} " :fixedarg\n")
	FILE(APPEND ${CONFIG_FILE} "\n")
      ENDIF(NB GREATER 0)

      FILE(APPEND ${CONFIG_FILE} " set xx=%CD% \n")
      FILE(APPEND ${CONFIG_FILE} " cd \"%${WHERE}%\\cmake\" \n")
      FILE(APPEND ${CONFIG_FILE} " if not exist CMakeCache.txt (\n")
      IF(MSVC)
	# -DCMAKE_BUILD_TYPE=Release added because it requires to export visual debug libraries for the distribution.
	FILE(APPEND ${CONFIG_FILE} "   cmake .. -G\"NMake Makefiles\" -D${CMD}=1 -DCMAKE_BUILD_TYPE=Release\n")
      ELSE(MSVC)
	# MinGW
	FILE(APPEND ${CONFIG_FILE} "   cmake .. -G\"MinGW Makefiles\" -D${CMD}=1 -DCMAKE_BUILD_TYPE=Release\n") 
      ENDIF(MSVC)
      FILE(APPEND ${CONFIG_FILE} " )\n")
      IF(IS_STD)
	FILE(APPEND ${CONFIG_FILE} " if not \"%arguments%\" == \"force\" (\n")
	FILE(APPEND ${CONFIG_FILE} " goto :not_force\n")
	FILE(APPEND ${CONFIG_FILE} " )\n")
	IF(MSVC)
	  FILE(APPEND ${CONFIG_FILE} "  nmake src_clean /i /s \n")
	  FILE(APPEND ${CONFIG_FILE} "  nmake clean /i /s \n")
	  FILE(APPEND ${CONFIG_FILE} "  nmake source /i /s \n")
	  FILE(APPEND ${CONFIG_FILE} "  nmake /i /s \n")
	ELSE(MSVC)
	  FILE(APPEND ${CONFIG_FILE} "  make src_clean -i -s \n")
	  FILE(APPEND ${CONFIG_FILE} "  make clean -i -s \n")
	  FILE(APPEND ${CONFIG_FILE} "  make source -i -s \n")
	  FILE(APPEND ${CONFIG_FILE} "  make -i -s \n")
	ENDIF(MSVC)

	FILE(APPEND ${CONFIG_FILE} "  goto :end1\n")
	FILE(APPEND ${CONFIG_FILE} " :not_force\n")
	IF(MSVC)
	  FILE(APPEND ${CONFIG_FILE} " nmake %arguments% /i /s \n")
	ELSE(MSVC)
	  FILE(APPEND ${CONFIG_FILE} " make %arguments% -i -s \n")
	ENDIF(MSVC)  
	FILE(APPEND ${CONFIG_FILE} " :end1\n")
      ELSEIF(IS_EXAMPLE)
	IF(MSVC)
	  FILE(APPEND ${CONFIG_FILE} " nmake %arguments% /i /s \n")
	ELSE(MSVC)
	  FILE(APPEND ${CONFIG_FILE} " make %arguments% -i -s \n")
	ENDIF(MSVC)
      ENDIF(IS_STD)
      FILE(APPEND ${CONFIG_FILE} " cd %xx% \n")
      FILE(APPEND ${CONFIG_FILE} " set xx= \n")
    ENDIF(UNIX) 
    ADD_EXECUTE_RIGHT(${CONFIG_FILE})
  ENDIF(PKCONFIG)
ENDMACRO(GEN_CMAKE_CMD CONFIG_FILE OPTIONS DEFAULT_OPTIONS CMD TYPE)

#------------------------------------------------------------------------------------------------------------
# It generates in CONFIG_FILE, the use of the command CMD parameterized by the options OPTIONS.
# The message MESSAGE describes the command CMD. It takes into account the tabulation environment 
# (see the references to the cmake variable NB_TABUL).
#
# Example: GEN_ECHO_USE(F makeSignalToolbox "source clean all doc" "to build a new to SignalToolbox package") 
#          will generate in F the following text (linux)
#          echo ">>>> use makeSignalToolbox [source clean all doc] to build a new SignalToolbox package"
#------------------------------------------------------------------------------------------------------------
MACRO(GEN_ECHO_USE CONFIG_FILE CMD OPTIONS MESSAGE)
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${_NAME_FILE_COMMANDS} "${CMD} ${OPTIONS} ${MESSAGE}\n")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_ECHO_USE  CONFIG_FILE CMD OPTIONS MESSAGE)

MACRO(ECHO_USE_HEADER_LINE CONFIG_FILE)
  GEN_ECHO (${CONFIG_FILE} "\"##--- POLYCHRONY INFO ---##--- POLYCHRONY INFO ---##--- POLYCHRONY INFO ---##---\"")
ENDMACRO(ECHO_USE_HEADER_LINE CONFIG_FILE)

MACRO(ECHO_USE_CMD_LINE CONFIG_FILE CMD_LINE)
  GEN_ECHO(${CONFIG_FILE} "\"${CMD_LINE}\"\n")
ENDMACRO(ECHO_USE_CMD_LINE CONFIG_FILE CMD_LINE)

MACRO(ECHO_USE_END_LINE CONFIG_FILE)
  GEN_ECHO (${CONFIG_FILE} "\"# ------------------# ------------------# ------------------# ---------------- #\"")
ENDMACRO(ECHO_USE_END_LINE CONFIG_FILE)

# --------------------------------------------------------------------
# Similar to GEN_ECHO_USE without the tabulations.
# --------------------------------------------------------------------
MACRO(GEN_ECHO_USE_CMD CONFIG_FILE CMD MESSAGE)
  ECHO_USE_HEADER_LINE(${CONFIG_FILE})
  GEN_ECHO(${CONFIG_FILE} "\"==> ${MESSAGE}\"\n")
  ECHO_USE_CMD_LINE(${CONFIG_FILE} ${CMD})
  ECHO_USE_END_LINE(${CONFIG_FILE})
ENDMACRO(GEN_ECHO_USE_CMD CONFIG_FILE CMD MESSAGE)


# -------------------------------------------------------------------------------------------------------------
# For Linux only, useful for pkMake tool.
# It generates in the file WHERE/FILENAME the script that defines the command echo
# Example: 
#    GEN_CMD_ECHO ${pK_com} echoWar WARNING 0) will generate the following lines in the file  ${pK_com}/echoWar
#    echo "%%  WARNING  %% $1"
#     if ( $# == 2 ) echo "              $2"
#     setenv xKStatus 0
# -------------------------------------------------------------------------------------------------------------
MACRO (GEN_CMD_ECHO WHERE FILENAME CODE STATUS_VALUE)
  IF(PKCONFIG)
    MK_CONFIG_NAME(${FILENAME} CONFIG_NAME)
    SET(CONFIG_FILE ${WHERE}/${CONFIG_NAME})
    GEN_HEADER_SCRIPT(${CONFIG_FILE})
    
    IF(UNIX)
      GEN_ECHO(${CONFIG_FILE} "%%  ${CODE}  %% $1")
      FILE(APPEND ${CONFIG_FILE} "if ( $# == 2 ) echo \"              $2\"\n")
      FILE(APPEND ${CONFIG_FILE} "setenv xKStatus ${STATUS_VALUE}")
    ENDIF(UNIX)
    ADD_EXECUTE_RIGHT(${CONFIG_FILE})
  ENDIF(PKCONFIG)
ENDMACRO (GEN_CMD_ECHO WHERE FILENAME CODE STATUS_VALUE)

MACRO(GEN_SCRIPT_TEST_ARG FILENAME TARGET)
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${FILENAME} "if (( \"$#\" != 0 ) && ( \"$1\" == \"${TARGET}\" )) then\n")
    ELSE(UNIX)
      FILE(APPEND ${FILENAME} " if \"%1%\" == \"${TARGET}\" (\n")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO (GEN_SCRIPT_TEST_ARG FILENAME TARGET)

MACRO(GEN_SCRIPT_EXIT FILENAME)
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${FILENAME} "exit -1\n")
    ELSE(UNIX)
      FILE(APPEND ${FILENAME} " goto :eof\n")    
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_SCRIPT_EXIT FILENAME)

MACRO(GEN_SCRIPT_END_TEST_ARG FILENAME TARGET)
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${FILENAME} "endif\n")
    ELSE(UNIX)
      FILE(APPEND ${FILENAME} " )\n")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_SCRIPT_END_TEST_ARG FILENAME TARGET)


#------------------------------------------------------------------
# It generates in the file CONFIG_FILE.
# - the test of the existence of the shell variable WHERE
# - the removing of the CMAKE cache in  WHERE/cmake directory;
# - the generation of the call to WHERE/CMD_NAME 
#------------------------------------------------------------------
MACRO(GEN_INSTALL_CMD CONFIG_FILE WHERE CMD_NAME)
  IF(PKCONFIG)
    GEN_REQUIRE_SHELL_VAR(${CONFIG_FILE} ${WHERE})
    
    IF (UNIX)
      FILE(APPEND ${CONFIG_FILE} " if ( -ef \"\${${WHERE}}/${CMD_NAME}\") then\n")
      FILE(APPEND ${CONFIG_FILE} "    \${${WHERE}}/${CMD_NAME} $*\n")
      FILE(APPEND ${CONFIG_FILE} " endif\n")
    ELSE (UNIX)
      FILE(APPEND ${CONFIG_FILE} " if exist \"%${WHERE}%\\${CMD_NAME}.bat\"  call \"%${WHERE}%\\${CMD_NAME}.bat\" %1%\n")
    ENDIF (UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_INSTALL_CMD CONFIG_FILE WHERE CMD_NAME)

MACRO(GEN_CMD_FIRSTCALL_DOCBUILDER CONFIG_FILE VFILE RMFILE EFILE)
  IF(PKCONFIG)
    IF (UNIX)
      FILE(APPEND ${CONFIG_FILE} " if (! -ef \"\${VFILE}\") then\n")
      FILE(APPEND ${CONFIG_FILE} "    rm -f \${RMFILE}\n")
      FILE(APPEND ${CONFIG_FILE} " endif\n")
      FILE(APPEND ${CONFIG_FILE} " if (! -ef \"\${EFILE}\") then\n")
      FILE(APPEND ${CONFIG_FILE} "    set BFIRSTIME=1\n")
      FILE(APPEND ${CONFIG_FILE} "  else \n")
      FILE(APPEND ${CONFIG_FILE} "    set BFIRSTIME=0\n")
      FILE(APPEND ${CONFIG_FILE} " endif\n")
      FILE(APPEND ${CONFIG_FILE} "labelDoc:\n")
    ELSE (UNIX)
      FILE(TO_NATIVE_PATH  ${VFILE} WINVFILE)
      FILE(TO_NATIVE_PATH  ${RMFILE} WINRMFILE)
      FILE(TO_NATIVE_PATH  ${EFILE} WINEFILE)
      FILE(APPEND ${CONFIG_FILE} " if not exist \"${WINVFILE}\"  del /S \"${WINRMFILE}\"\n")
      FILE(APPEND ${CONFIG_FILE} " if not exist \"%${WINEFILE}%\"  (set BFIRSTIME=1) ELSE (set BFIRSTIME=0)\n")
      FILE(APPEND ${CONFIG_FILE} ":labelDoc\n")
    ENDIF (UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_CMD_FIRSTCALL_DOCBUILDER CONFIG_FILE VFILE RMFILE)

MACRO(GEN_PRINT_FILE CONFIG_FILE DIR FILE)
  IF (UNIX)
    FILE(APPEND ${CONFIG_FILE} "more ${DIR}/${FILE}\n")
  ELSE (UNIX)
    FILE(TO_NATIVE_PATH ${DIR} NATIVE_DIR)
    FILE(APPEND ${CONFIG_FILE} "more \"${NATIVE_DIR}\\${FILE}\"\n")
  ENDIF (UNIX)
ENDMACRO(GEN_PRINT_FILE CONFIG_FILE DIR FILE)


MACRO(GEN_LIST_MAKE CONFIG_FILE)
  GEN_SCRIPT_TEST_ARG(${CONFIG_FILE} "-l")
  GEN_PRINT_FILE(${CONFIG_FILE} ${pK_ROOT} _cmd_makePolychronyToolset)
  GEN_SCRIPT_EXIT(${CONFIG_FILE})
  GEN_SCRIPT_END_TEST_ARG(${CONFIG_FILE} "-l")
ENDMACRO(GEN_LIST_MAKE CONFIG_FILE)

MACRO(GEN_HELP_MAKE CONFIG_FILE)
  GEN_SCRIPT_TEST_ARG(${CONFIG_FILE} "-h")
  GEN_PRINT_FILE(${CONFIG_FILE} ${pK_ROOT}/com docMake.txt)
  GEN_SCRIPT_EXIT(${CONFIG_FILE})
  GEN_SCRIPT_END_TEST_ARG(${CONFIG_FILE} "-h")
ENDMACRO(GEN_HELP_MAKE CONFIG_FILE)

MACRO(GEN_CMD_FIRSTCALL_DOCBUILDER_END CONFIG_FILE)
  IF(PKCONFIG)
    IF (UNIX)
      FILE(APPEND ${CONFIG_FILE} " if (\"$BFIRSTIME\" == \"1\") then\n")
      FILE(APPEND ${CONFIG_FILE} "    $pK_com/pk_clean_doc\n")
      FILE(APPEND ${CONFIG_FILE} "    set BFIRSTIME=0\n")
      FILE(APPEND ${CONFIG_FILE} "    goto labelDoc\n")
      FILE(APPEND ${CONFIG_FILE} " endif\n")
    ELSE (UNIX)
      FILE(APPEND ${CONFIG_FILE} " if \"%BFIRSTIME%\" == \"1\" (\n")
      FILE(APPEND ${CONFIG_FILE} "    set BFIRSTIME=0\n")
      FILE(APPEND ${CONFIG_FILE} "    call %pK_com%/pk_clean_doc\n")
      FILE(APPEND ${CONFIG_FILE} "    goto labelDoc\n")
      FILE(APPEND ${CONFIG_FILE} " )\n")
    ENDIF (UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_CMD_FIRSTCALL_DOCBUILDER_END CONFIG_FILE)


# ---------------------------------------------------------------------------------------------------------------------------------
# It generates in the file ${${ADIR}}/${CMD} a command CMD parameterized by the options OPTIONS (default option: DEFAULT_OPTIONS).
#   - the requirement of the shell variables of the list VARS_REQUIRED is generated (see GEN_REQUIRE_SHELL_VAR())
#   - the requirement of the availability of the the directories of the list DIRS_VARS_REQUIRED is generated (see GEN_REQUIRE_DIR())
#   
# It returns in CMD_FILE the value of ${${ADIR}}/${CMD} (os dependant).
# ----------------------------------------------------------------------------------------------------------------------------------
MACRO(GEN_CMD_FILE_LOC ADIR VARS_REQUIRED DIRS_VAR_REQUIRED OPTIONS DEFAULT_OPTIONS CMD CMD_FILE TYPE)

  SET(LOC_FILE ${${ADIR}}/${CMD})

  SET(CURRENT_DEFS_CMAKE ${${ADIR}}/${PREDEF_DEFS_FILENAME_CMAKE})
  GEN_HEADER_SCRIPT(${LOC_FILE})
  SET(CURRENT_DEFS_CMAKE)

  FOREACH(VAR ${VARS_REQUIRED})
    GEN_REQUIRE_SHELL_VAR(${LOC_FILE} ${VAR})
  ENDFOREACH(VAR)
  
  GEN_HELP_MAKE(${LOC_FILE})
  GEN_LIST_MAKE(${LOC_FILE})
  FOREACH(VAR ${DIRS_VAR_REQUIRED})
    GEN_REQUIRE_DIR(${LOC_FILE} ${VAR})
  ENDFOREACH(VAR)

  GEN_CMAKE_CMD(${LOC_FILE} "${OPTIONS}" "${DEFAULT_OPTIONS}" ${CMD}  ${ADIR} ${TYPE})

  SET(${CMD_FILE} ${LOC_FILE})
ENDMACRO(GEN_CMD_FILE_LOC ADIR VARS_REQUIRED DIRS_VAR_REQUIRED OPTIONS DEFAULT_OPTIONS CMD  CMD_FILE TYPE)


# ---------------------------------------------------------------------------------------------------------------------------------
# It generates in the file ${${ADIR}}/${CMD} a command CMD parameterized by the options OPTIONS (default option: DEFAULT_OPTIONS).
#   - the requirement of the shell variables of the list VARS_REQUIRED is generated (see GEN_REQUIRE_SHELL_VAR())
#   - the requirement of the availability of the the directories of the list DIRS_VARS_REQUIRED is generated (see GEN_REQUIRE_DIR())
#   
# It returns in CMD_FILE the value of ${${ADIR}}/${CMD} (os dependant).
# ----------------------------------------------------------------------------------------------------------------------------------
MACRO(GEN_CMD_FILE ADIR VARS_REQUIRED DIRS_VAR_REQUIRED OPTIONS DEFAULT_OPTIONS CMD CMD_FILE)

  GEN_CMD_FILE_LOC("${ADIR}" "${VARS_REQUIRED}" "${DIRS_VAR_REQUIRED}" "${OPTIONS}" "${DEFAULT_OPTIONS}" "${CMD}" "${CMD_FILE}" STD)

ENDMACRO(GEN_CMD_FILE ADIR VARS_REQUIRED DIRS_VAR_REQUIRED OPTIONS DEFAULT_OPTIONS CMD  CMD_FILE)

# -------------------------------------------------------------------------------------
# It returns in NAMES the cmake string of the name files of a cmake list LIST. 
# LIST is assumed to be a list of absolute path.
# -------------------------------------------------------------------------------------
MACRO(EXTRACT_FILE_NAMES LIST NAMES)
  SET(YY)
  FOREACH(AFILE ${LIST})
    GET_FILENAME_COMPONENT(ELEM ${AFILE} NAME)
    SET(YY "${YY} ${ELEM},")
  ENDFOREACH(AFILE)
  STRING(REGEX REPLACE ",$" "." ZZ "${YY}")
  SET(${NAMES} ${ZZ})
ENDMACRO(EXTRACT_FILE_NAMES LIST NAMES)


# ---------------------------------------------------------------------------------
# It defines the copying a file from INPUTDIR/FILENAME to OUTPUTDIR/FILENAME_TARGET
# with the dependence INPUTDIR/FILENAME --> OUTPUTDIR/FILENAME_TARGET
# ----------------------------------------------------------------------------------
MACRO(COPY_FILE_GEN INPUTDIR OUTPUTDIR FILENAME FILENAME_TARGET )
  ADD_CUSTOM_COMMAND(
    OUTPUT ${OUTPUTDIR}/${FILENAME_TARGET}
    COMMAND ${CMAKE_COMMAND} -E echo "Copying...${INPUTDIR}/${FILENAME} to ${OUTPUTDIR}"
    COMMAND ${CMAKE_COMMAND} -E copy ${INPUTDIR}/${FILENAME} ${OUTPUTDIR}/${FILENAME_TARGET}
    WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
    DEPENDS ${INPUTDIR}/${FILENAME}
    )
ENDMACRO(COPY_FILE_GEN INPUTDIR OUTPUTDIR FILENAME FILENAME_TARGET)

# ---------------------------------------------------------------------------------
# It defines the copying a file from INPUTDIR/FILENAME to OUTPUTDIR/FILENAME_TARGET
# with the dependence INPUTDIR/FILENAME --> OUTPUTDIR/FILENAME_TARGET
# ----------------------------------------------------------------------------------
MACRO(MUST_BE_UPDATED INPUTDIR OUTPUTDIR FILENAME FILENAME_TARGET )
  ADD_CUSTOM_COMMAND(
    OUTPUT ${OUTPUTDIR}/${FILENAME_TARGET}
    COMMAND ${CMAKE_COMMAND} -E echo "The File ${OUTPUTDIR}/${FILENAME_TARGET} MUST BE UPDATED FROM ${INPUTDIR}/${FILENAME}"
    WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
    DEPENDS ${INPUTDIR}/${FILENAME}
    )
ENDMACRO(MUST_BE_UPDATED INPUTDIR OUTPUTDIR FILENAME FILENAME_TARGET)

# --------------------------------------------------------------------------
# It defines the copying a file from INPUTDIR/FILENAME to OUTPUTDIR/FILENAME
# --------------------------------------------------------------------
MACRO(COPY_FILE INPUTDIR OUTPUTDIR FILENAME)
  COPY_FILE_GEN("${INPUTDIR}" "${OUTPUTDIR}" "${FILENAME}" "${FILENAME}" )
ENDMACRO(COPY_FILE INPUTDIR OUTPUTDIR FILENAME)

MACRO(SET_STATIC_BINARY EXE_NAME)
  IF(NOT APPLE)
    IF(UNIX)
      # SET_TARGET_PROPERTIES(${EXE_NAME}  PROPERTIES LINK_FLAGS "-static")
    ENDIF(UNIX)
  ENDIF(NOT APPLE)
ENDMACRO(SET_STATIC_BINARY EXE_NAME)


# ---------------------------------------------------------------------------------------------------
# It adds, if the variable VAR1 is not yet defined, to the config file of the directory DIRFROM the 
# definition of the variable VAR1 by the value VALUE.
# ---------------------------------------------------------------------------------------------------
MACRO(USER_DEF_VALUE DIRFROM VAR1 VALUE)
  IF(NOT DEFINED ${VAR1})
    GET_CONFIG_FILE(${DIRFROM} NAME_FILE_CONFIG) 
    ADD_SHELL_VAR(${NAME_FILE_CONFIG} ${VAR1} ${VALUE} "")
    SET(${VAR1} ${VALUE})
  ENDIF(NOT DEFINED ${VAR1})
ENDMACRO(USER_DEF_VALUE DIRFROM VAR1 VAR2)

# ---------------------------------------------------------------------------------------------------
# It adds, if the variable VAR1 is not yet defined, to the config file of the directory DIRFROM the 
# definition of the variable VAR1 by the value of the variable VAR2
# ---------------------------------------------------------------------------------------------------
MACRO(USER_DEF_VAR DIRFROM VAR1 VAR2)
  IF(NOT DEFINED ${VAR1})
    #      IF(DEFINED ${VAR2})
    MAKE_REF_SHELLVAR(${VAR2} NAME)
    USER_DEF_VALUE( ${DIRFROM} ${VAR1} ${NAME})
    SET(${VAR1} ${${VAR2}})
    #      ENDIF(DEFINED ${VAR2})
  ENDIF(NOT DEFINED ${VAR1})
ENDMACRO(USER_DEF_VAR DIRFROM VAR1 VAR2)

# ---------------------------------------------------------------------------------------------------
# It resets the global config file GLOBAL_CONFIG_FILE. It becomes undefined for CMAKE.
# ---------------------------------------------------------------------------------------------------
MACRO(RESET_CONFIG_FILE)
  SET(GLOBAL_CONFIG_FILE)
ENDMACRO(RESET_CONFIG_FILE)
# ---------------------------------------------------------------------------------------------------
# It defines the global config file  to VFILECONFIG
# ---------------------------------------------------------------------------------------------------
MACRO(SET_CONFIG_FILE_TO XVFILECONFIG)
  MK_CONFIG_NAME(${XVFILECONFIG} VFILECONFIG)
  SET(GLOBAL_CONFIG_FILE ${VFILECONFIG})
  IF(PKCONFIG)
    FILE(REMOVE ${GLOBAL_CONFIG_FILE})
    BEGIN_CONFIG_FILE(${GLOBAL_CONFIG_FILE})
  ENDIF(PKCONFIG)
ENDMACRO(SET_CONFIG_FILE_TO VCONFIG)

# ---------------------------------------------------------------------------------------------------
# Adding, in the config file of DIRFROM, the definition of the symbol CMD by WHERE/BINNAME
# ---------------------------------------------------------------------------------------------------
MACRO(DEF_COMMAND DIRFROM WHERE BINNAME CMD)
  IF(PKCONFIG)
    GET_CONFIG_FILE(${DIRFROM} NAME_FILE_CONFIG)
    MAKE_REF_SHELLVAR(${WHERE} NAME1)
    ADD_PATH_SEPARATOR(${NAME1} NAME1)
    ADD_SHELL_VAR(${NAME_FILE_CONFIG} ${CMD} ${NAME1}${BINNAME} "")
  ENDIF(PKCONFIG)
ENDMACRO(DEF_COMMAND DIRFROM WHERE  BINNAME CMD)

# ---------------------------------------------------------------------------------------------------
# Adding a dependence
# FOO_MOD/FOO.MODSUFF depends on FOO_SRC/FOO_g.SRCSUFF, FOO_SRC/FOO_m.SRCSUFF + OTHERS
# ---------------------------------------------------------------------------------------------------
MACRO(ADD_PK_DEPENDENCE FOO_SRC FOO_MOD FOO SRCSUFF MODSUFF OTHERS)
  SET(X ${FOO_SRC}/${FOO}_g.${SRCSUFF} ${FOO_SRC}/${FOO}_m.${SRCSUFF} ${OTHERS})
  SET_SOURCE_FILES_PROPERTIES(${FOO_MOD}/${FOO}.${MODSUFF} PROPERTIES OBJECT_DEPENDS "${X}")
ENDMACRO(ADD_PK_DEPENDENCE FOO_SRC FOO_MOD FOO)

# -------------------------------------------------------------------------
# Finalizing a library PACKAGE
# -------------------------------------------------------------------------
MACRO(LIBRARY_FINALIZING PACKAGE)
  # On Mac, common symbols not allowed with MH_DYLIB output format with the -multi_module option
  IF(APPLE)
# SET_TARGET_PROPERTIES(${PACKAGE} PROPERTIES SUFFIX ".jnilib" LINK_FLAGS "-single_module")
# modified from JDK 1.6 to JDK 1.7
    SET_TARGET_PROPERTIES(${PACKAGE} PROPERTIES SUFFIX ".dylib" LINK_FLAGS "-single_module")
  ENDIF(APPLE)
ENDMACRO(LIBRARY_FINALIZING PACKAGE)

#---------------------------------------------------------------------------
# UNUSED
#---------------------------------------------------------------------------
MACRO(SETEQUIVxx NAME1 NAME2 DIR)
  IF (PKCONFIG)
    SET(NAME_FILE_CONFIG ${DIR}/${PK_CONFIG_FILE})
    MAKE_REF_SHELLVAR(${NAME2} NAME)
    ADD_SHELL_VAR(${NAME_FILE_CONFIG} ${NAME1} "${NAME}" "")
  ENDIF (PKCONFIG)
ENDMACRO(SETEQUIVxx NAME1 NAME2 DIR)


# Send the message MESS to the console
FUNCTION(pK_ERROR MESS)
  MESSAGE(${MESS})
ENDFUNCTION()


# It returns in the variable PK_GET_DIR_NAME, the value of the cmake variable VarDIR (or the value of
# the shell variable called VarDIR). The value is assumed to be the path of a directory. 
# If the directory doest not exist the variable (PK_GET_DIR_NAME) is not defined and an error is send to the console.
FUNCTION(GET_DIR_NAME VarDIR PK_GET_DIR_NAME)
  SET(SRCDIR ${${VarDIR}})
  IF(NOT SRCDIR)
    SET(SRCDIR "$ENV{${VarDIR}}")
    PRINT(" +++ ENV{${VarDIR}} = ${SRCDIR}")
  ELSE(NOT SRCDIR)
    PRINT(" °°° ${VarDIR} = ${SRCDIR}")
  ENDIF(NOT SRCDIR)

  IF (EXISTS ${SRCDIR})
    SET(${PK_GET_DIR_NAME} ${SRCDIR} PARENT_SCOPE)
    PRINT("GET_DIR_NAME, ${PK_GET_DIR_NAME} TO ${${PK_GET_DIR_NAME}}")
  ELSE()
    PK_ERROR("GET_DIR_NAME (1), ${MESS1} ${SRCDIR}, PARAM= ${VarDIR}")
    SET(${PK_GET_DIR_NAME} PARENT_SCOPE)
  ENDIF()
ENDFUNCTION()

FUNCTION(GET_SYSTEM_INFOS VSYSTEM_NAME  VSYSTEM_VERSION)

  SET(${VSYSTEM_VERSION} ${CMAKE_SYSTEM_PROCESSOR} PARENT_SCOPE)
  IF(APPLE)
    SET(XX macosx)
  ELSE(APPLE)
    IF(UNIX)
      SET(XX linux)
    ELSE(UNIX)
      SET(XX win32)
      IF(MSVC)
	SET(${VSYSTEM_VERSION} ${MSVC_C_ARCHITECTURE_ID} PARENT_SCOPE)
      ENDIF(MSVC)
    ENDIF(UNIX)
  ENDIF(APPLE)
  PRINT(" ----------------------- OS = ${XX}")
  SET(${VSYSTEM_NAME} ${XX} PARENT_SCOPE)
  PRINT(" ----------------------- PROCESSOR = ${CMAKE_SYSTEM_PROCESSOR}")
ENDFUNCTION()

# generate cd ${VAR}/DIR in CONFIG_FILE
FUNCTION(GEN_GOTO_DIR_DIR CONFIG_FILE VAR DIR)
  MAKE_REF_SHELLVAR(${VAR} NAME)
  ADD_PATH_SEPARATOR(${NAME} NAME)
  SET(NAME "${NAME}${DIR}")
  FILE(APPEND ${CONFIG_FILE} "cd ${NAME} \n")

ENDFUNCTION(GEN_GOTO_DIR_DIR CONFIG_FILE VAR DIR)


FUNCTION(GEN_DELETE CONFIG_FILE TARGET)
  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} "\\rm -f -R ${TARGET}/* \n")
  ELSE(UNIX)
    FILE(TO_NATIVE_PATH ${TARGET} NATIVE_TARGET)
    FILE(APPEND ${CONFIG_FILE} "del /Q/S ${NATIVE_TARGET}\\* \n")
  ENDIF(UNIX)
ENDFUNCTION()

# Export
FUNCTION (GEN_EXPORT_FILE SRC_DIR SRCFILE )
  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} "cp \${${SRC_DIR}}/${SRCFILE} \${pK_export}/${SRCFILE}\n")
  ELSE(UNIX)
    FILE(TO_NATIVE_PATH ${${SRC_DIR}} NATIVE_SRC_DIR)
    FILE(TO_NATIVE_PATH ${pK_export} NATIVE_pK_export)
    FILE(APPEND ${CONFIG_FILE} "copy ${NATIVE_SRC_DIR}\\${SRCFILE} ${NATIVE_pK_export}\\${SRCFILE}\n")
  ENDIF(UNIX)
ENDFUNCTION()

# Export
FUNCTION (GEN_MOVE_RES CONFIG_FILE VAR_WHERE GENERATEDTAR CPACKGEN)
  SET (C_SUFF "tar.gz")  # TGZ
  IF(${CPACKGEN} STREQUAL "NSIS")
    SET (C_SUFF "exe")
  ENDIF(${CPACKGEN} STREQUAL "NSIS")

  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} "mv \${${VAR_WHERE}}/export/${GENERATEDTAR}.${C_SUFF} \${pK_export}/${GENERATEDTAR}.${C_SUFF}\n")
  ELSE(UNIX)
    FILE(APPEND ${CONFIG_FILE} "move %${VAR_WHERE}%\\export\\${GENERATEDTAR}.${C_SUFF} %pK_export%\\${GENERATEDTAR}.${C_SUFF}\n")
  ENDIF(UNIX)
ENDFUNCTION()

# Export
FUNCTION(GEN_DEF_HERE CONFIG_FILE)
  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} "set HERE=`pwd`\n")
  ELSE(UNIX)
    FILE(APPEND ${CONFIG_FILE} "set HERE=%CD%\n")
  ENDIF(UNIX)
ENDFUNCTION()

# Export
FUNCTION(GEN_CHDIR_HERE CONFIG_FILE)
  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} "cd \$HERE\n")
  ELSE(UNIX)
    FILE(APPEND ${CONFIG_FILE} "cd %HERE%\n")
  ENDIF(UNIX)
ENDFUNCTION()

# Export
FUNCTION(GEN_CLEAN_EXPORTDIR CONFIG_FILE VAR_WHERE)
  IF(UNIX)
    FILE(APPEND ${CONFIG_FILE} "rm -R -f \${${VAR_WHERE}}/export\n")
  ELSE(UNIX)
    FILE(APPEND ${CONFIG_FILE} "del /Q/S %${VAR_WHERE}%\\export > xxx.log\n")
  ENDIF(UNIX)
ENDFUNCTION()


FUNCTION(INCLUDE_XML_DIRECTORY)
 #  IF(UNIX)
 #     INCLUDE_DIRECTORIES(/usr/include/libxml2)
 #   ENDIF(UNIX)
ENDFUNCTION()



#------------------------------------------------------------------------------------------------------
# It generates in the file VFILE the requirement of a shell variable pK_ROOT.
# If pK_ROOT is not set it generates the call to the setup to Polychrony.
#------------------------------------------------------------------------------------------------------
MACRO(GEN_REQUIRE_PK_ROOT VFILE )
  IF(PKCONFIG)
    IF(UNIX)
      FILE(APPEND ${VFILE} " if ( ! \$?pK_ROOT ) then \n")
      FILE(APPEND ${VFILE} "   set cmd_dir=`readlink -f $0`\n")
      FILE(APPEND ${VFILE} "   set cmd_dir=`dirname $cmd_dir`\n")
      FILE(APPEND ${VFILE} "   set pk_dir=`readlink -f $cmd_dir/..`\n")
      FILE(APPEND ${VFILE} "   setenv pK_ROOT $pk_dir\n")
      GEN_REQUIRE_FILE(${VFILE} pK_ROOT "PolychronyToolset_setup")
      FILE(APPEND ${VFILE} "   source $pK_ROOT/PolychronyToolset_setup >& /dev/null\n")
      FILE(APPEND ${VFILE} " endif\n")
      FILE(APPEND ${VFILE} "\n")
    ELSE(UNIX) #WIN32
      FILE(APPEND ${SIGNAL_CMD} "if not \"%pK_ROOT%\" == \"\" goto PK_ROOT_OK\n")
      FILE(APPEND ${SIGNAL_CMD} "\n")
      FILE(APPEND ${SIGNAL_CMD} "rem %~dp0 is the path of the current file (when it runs)\n")
      FILE(APPEND ${SIGNAL_CMD} "set pK_ROOT=\"%~dp0..\n")
      GEN_REQUIRE_FILE(${VFILE} pK_ROOT "PolychronyToolset_setup.bat")
      FILE(APPEND ${SIGNAL_CMD} "call \"%pK_ROOT%\\PolychronyToolset_setup.bat\"\n")
      FILE(APPEND ${SIGNAL_CMD} "\n")
      FILE(APPEND ${SIGNAL_CMD} ":PK_ROOT_OK\n")
    ENDIF(UNIX)
  ENDIF(PKCONFIG)
ENDMACRO(GEN_REQUIRE_PK_ROOT VFILE)

